import ast
import pathlib
import re
import sys

CODE_ROOT = pathlib.Path(__file__).resolve().parent.parent
EXECUTION_MODULES_PATH = CODE_ROOT / "src" / "saltext" / "namecheap" / "modules"


def check_cli_examples(files):
    """
    Check that every function on every execution module provides a CLI example
    """
    errors = 0
    for file in files:
        path = pathlib.Path(file).resolve()
        try:
            relpath = path.relative_to(EXECUTION_MODULES_PATH)
            if str(relpath.parent) != ".":
                # We don't want to check nested packages
                continue
        except ValueError:
            # We're only interested in execution modules
            continue
        module = ast.parse(path.read_text(), filename=str(path))
        for funcdef in [node for node in module.body if isinstance(node, ast.FunctionDef)]:
            if funcdef.name.startswith("_"):
                # We're not interested in internal functions
                continue

            docstring = ast.get_docstring(funcdef, clean=False)
            if not docstring:
                errors += 1
                print(
                    "The function {!r} on '{}' does not have a docstring".format(
                        funcdef.name,
                        path.relative_to(CODE_ROOT),
                    ),
                    file=sys.stderr,
                )
                continue

            if _check_cli_example_present(docstring) is False:
                errors += 1
                print(
                    "The function {!r} on '{}' does not have a 'CLI Example:' in it's docstring".format(
                        funcdef.name,
                        path.relative_to(CODE_ROOT),
                    ),
                    file=sys.stderr,
                )
                continue
    sys.exit(errors)


CLI_EXAMPLE_PRESENT_RE = re.compile(r"CLI Example(?:s)?:")


def _check_cli_example_present(docstring):
    return CLI_EXAMPLE_PRESENT_RE.search(docstring) is not None


if __name__ == "__main__":
    check_cli_examples(sys.argv[1:])



"""
Namecheap DNS Management

.. versionadded:: 2017.7.0

Prerequisites
-------------

This module uses the ``requests`` Python module to communicate to the namecheap
API.

Configuration
-------------

The Namecheap username, API key and URL should be set in the minion configuration
file, or in the Pillar data.

.. code-block:: yaml

    namecheap.name: companyname
    namecheap.key: a1b2c3d4e5f67a8b9c0d1e2f3
    namecheap.client_ip: 162.155.30.172
    #Real url
    namecheap.url: https://api.namecheap.com/xml.response
    #Sandbox url
    #namecheap.url: https://api.sandbox.namecheap.xml.response
"""

from saltext.namecheap.utils import namecheap

__virtualname__ = "namecheap_domains_dns"


def __virtual__():
    """
    Check to make sure requests and xml are installed and requests
    """
    return __virtualname__


def get_hosts(sld, tld):
    """
    Retrieves DNS host record settings for the requested domain.

    returns a dictionary of information about the requested domain

    sld
        SLD of the domain name

    tld
        TLD of the domain name

    CLI Example:

    .. code-block:: bash

        salt 'my-minion' namecheap_domains_dns.get_hosts sld tld
    """
    opts, url = namecheap.get_opts(__salt__["config.option"], "namecheap.domains.dns.gethosts")
    opts["TLD"] = tld
    opts["SLD"] = sld

    response_xml = namecheap.get_request(url, opts)
    if response_xml is None:
        return {}

    domaindnsgethostsresult = response_xml.getElementsByTagName("DomainDNSGetHostsResult")[0]

    return namecheap.xml_to_dict(domaindnsgethostsresult)


def get_list(sld, tld):
    """
    Gets a list of DNS servers associated with the requested domain.

    returns a dictionary of information about requested domain

    sld
        SLD of the domain name

    tld
        TLD of the domain name

    CLI Example:

    .. code-block:: bash

        salt 'my-minion' namecheap_domains_dns.get_list sld tld
    """
    opts, url = namecheap.get_opts(__salt__["config.option"], "namecheap.domains.dns.getlist")
    opts["TLD"] = tld
    opts["SLD"] = sld

    response_xml = namecheap.get_request(url, opts)
    if response_xml is None:
        return {}

    domaindnsgetlistresult = response_xml.getElementsByTagName("DomainDNSGetListResult")[0]

    return namecheap.xml_to_dict(domaindnsgetlistresult)


def set_hosts(sld, tld, hosts):
    """
    Sets DNS host records settings for the requested domain.

    returns True if the host records were set successfully

    sld
        SLD of the domain name

    tld
        TLD of the domain name

    hosts
        Must be passed as a list of Python dictionaries, with each dictionary
        containing the following keys:

        - **hostname**
        - **recordtype** - One of ``A``, ``AAAA``, ``CNAME``, ``MX``, ``MXE``,
          ``TXT``, ``URL``, ``URL301``, or ``FRAME``
        - **address** - URL or IP address
        - **ttl** - An integer between 60 and 60000 (default: ``1800``)

        Additionally, the ``mxpref`` key can be present, but must be accompanied
        by an ``emailtype`` key.

    CLI Example:

    .. code-block:: bash

        salt 'my-minion' namecheap_domains_dns.set_hosts sld tld hosts
    """
    opts, url = namecheap.get_opts(__salt__["config.option"], "namecheap.domains.dns.setHosts")
    opts["SLD"] = sld
    opts["TLD"] = tld
    i = 1
    for hostrecord in hosts:
        str_i = str(i)
        opts["HostName" + str_i] = hostrecord["hostname"]
        opts["RecordType" + str_i] = hostrecord["recordtype"]
        opts["Address" + str_i] = hostrecord["address"]
        if "ttl" in hostrecord:
            opts["TTL" + str_i] = hostrecord["ttl"]
        if "mxpref" in hostrecord:
            opts["MXPref" + str_i] = hostrecord["mxpref"]
            opts["EmailType"] = hostrecord["emailtype"]
        i += 1

    response_xml = namecheap.post_request(url, opts)
    if response_xml is None:
        return False

    dnsresult = response_xml.getElementsByTagName("DomainDNSSetHostsResult")[0]
    return namecheap.string_to_value(dnsresult.getAttribute("IsSuccess"))


def set_custom(sld, tld, nameservers):
    """
    Sets domain to use custom DNS servers.

    returns True if the custom nameservers were set successfully

    sld
        SLD of the domain name

    tld
        TLD of the domain name

    nameservers
        array of strings  List of nameservers to be associated with this domain

    CLI Example:

    .. code-block:: bash

        salt 'my-minion' namecheap_domains_dns.set_custom sld tld nameserver
    """
    opts, url = namecheap.get_opts(__salt__["config.option"], "namecheap.domains.dns.setCustom")
    opts["SLD"] = sld
    opts["TLD"] = tld
    opts["Nameservers"] = ",".join(nameservers)
    response_xml = namecheap.post_request(url, opts)
    if response_xml is None:
        return False

    dnsresult = response_xml.getElementsByTagName("DomainDNSSetCustomResult")[0]
    return namecheap.string_to_value(dnsresult.getAttribute("Update"))


def set_default(sld, tld):
    """
    Sets domain to use namecheap default DNS servers. Required for free
    services like Host record management, URL forwarding, email forwarding,
    dynamic DNS and other value added services.

    sld
        SLD of the domain name

    tld
        TLD of the domain name

    Returns ``True`` if the domain was successfully pointed at the default DNS
    servers.

    CLI Example:

    .. code-block:: bash

        salt 'my-minion' namecheap_domains_dns.set_default sld tld
    """
    opts, url = namecheap.get_opts(__salt__["config.option"], "namecheap.domains.dns.setDefault")
    opts["SLD"] = sld
    opts["TLD"] = tld
    response_xml = namecheap.post_request(url, opts)
    if response_xml is None:
        return False

    dnsresult = response_xml.getElementsByTagName("DomainDNSSetDefaultResult")[0]
    return namecheap.string_to_value(dnsresult.getAttribute("Updated"))



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html
# -- Path setup --------------------------------------------------------------
# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import datetime
import email.policy
import os
import sys
from pathlib import Path

try:
    from importlib_metadata import distribution
except ImportError:
    from importlib.metadata import distribution


try:
    docs_basepath = os.path.abspath(os.path.dirname(__file__))
except NameError:
    # sphinx-intl and six execute some code which will raise this NameError
    # assume we're in the doc/ directory
    docs_basepath = os.path.abspath(os.path.dirname("."))

PROJECT_ROOT_DIR = Path(docs_basepath).parent

addtl_paths = (
    os.path.join(os.pardir, "src"),  # saltext.namecheap itself (for autodoc)
    "_ext",  # custom Sphinx extensions
)

for addtl_path in addtl_paths:
    sys.path.insert(0, os.path.abspath(os.path.join(docs_basepath, addtl_path)))

dist = distribution("saltext.namecheap")


# -- Project information -----------------------------------------------------
this_year = datetime.datetime.today().year
if this_year == 2023:
    copyright_year = "2023"
else:
    copyright_year = f"2023 - {this_year}"
project = dist.metadata["Summary"]
author = dist.metadata.get("Author")

if author is None:
    # Core metadata is serialized differently with pyproject.toml:
    # https://packaging.python.org/en/latest/specifications/pyproject-toml/#authors-maintainers
    author_email = dist.metadata["Author-email"]
    em = email.message_from_string(
        f"To: {author_email}",
        policy=email.policy.default,
    )
    if em["To"].addresses and em["To"].addresses[0]:
        author = em["To"].addresses[0].display_name
    author = author or ""

copyright = f"{copyright_year}, {author}"

# The full version, including alpha/beta/rc tags
release = dist.version


# Variables to pass into the docs from sitevars.rst for rst substitution
with open("sitevars.rst") as site_vars_file:
    site_vars = site_vars_file.read().splitlines()

rst_prolog = """
{}
""".format(
    "\n".join(site_vars[:])
)

# -- General configuration ---------------------------------------------------

linkcheck_ignore = [r"http://localhost:\d+"]

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.napoleon",
    "sphinx.ext.intersphinx",
    "sphinx.ext.viewcode",
    "sphinx.ext.todo",
    "sphinx.ext.coverage",
    "sphinx_copybutton",
    "sphinxcontrib.spelling",
    "saltdomain",
    "sphinxcontrib.towncrier.ext",
    "myst_parser",
    "sphinx_inline_tabs",
]

myst_enable_extensions = [
    "colon_fence",
    "deflist",
    "tasklist",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = [
    "_build",
    "Thumbs.db",
    ".DS_Store",
    ".vscode",
    ".venv",
    ".git",
    ".gitlab-ci",
    ".gitignore",
    "sitevars.rst",
]

autosummary_generate = False

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "furo"
html_title = project

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = ""

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large. Favicons can be up to at least 228x228. PNG
# format is supported as well, not just .ico'
html_favicon = ""

# Sphinx Napoleon Config
napoleon_google_docstring = True
napoleon_numpy_docstring = False
napoleon_include_init_with_doc = True
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = False
napoleon_use_admonition_for_notes = False
napoleon_use_admonition_for_references = False
napoleon_use_ivar = False
napoleon_use_param = True
napoleon_use_rtype = True

# ----- Intersphinx Config ---------------------------------------------------------------------------------------->
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "pytest": ("https://docs.pytest.org/en/stable", None),
    "salt": ("https://docs.saltproject.io/en/latest", None),
}
# <---- Intersphinx Config -----------------------------------------------------------------------------------------

# ----- Autodoc Config ---------------------------------------------------------------------------------------------->
autodoc_default_options = {"member-order": "bysource"}
autodoc_mock_imports = ["salt"]
# <---- Autodoc Config -----------------------------------------------------------------------------------------------

# Towncrier draft config
towncrier_draft_autoversion_mode = "sphinx-release"
towncrier_draft_include_empty = True
towncrier_draft_working_directory = str(PROJECT_ROOT_DIR)


def setup(app):
    app.add_crossref_type(
        directivename="fixture",
        rolename="fixture",
        indextemplate="pair: %s; fixture",
    )
    # Allow linking to pytest's confvals.
    app.add_object_type(
        "confval",
        "pytest-confval",
        objname="configuration value",
        indextemplate="pair: %s; configuration value",
    )



import logging
import os

import pytest
from saltfactories.utils import random_string

from saltext.namecheap import PACKAGE_ROOT

# Reset the root logger to its default level(because salt changed it)
logging.root.setLevel(logging.WARNING)


# This swallows all logging to stdout.
# To show select logs, set --log-cli-level=<level>
for handler in logging.root.handlers[:]:
    logging.root.removeHandler(handler)
    handler.close()


@pytest.fixture(scope="session")
def salt_factories_config():
    """
    Return a dictionary with the keyword arguments for FactoriesManager
    """
    return {
        "code_dir": str(PACKAGE_ROOT),
        "inject_sitecustomize": "COVERAGE_PROCESS_START" in os.environ,
        "start_timeout": 120 if os.environ.get("CI") else 60,
    }


@pytest.fixture(scope="package")
def master(salt_factories):
    return salt_factories.salt_master_daemon(random_string("master-"))


@pytest.fixture(scope="package")
def minion(master):
    return master.salt_minion_daemon(random_string("minion-"))



import ast
import os.path
import subprocess
from pathlib import Path

repo_path = Path(subprocess.check_output(["git", "rev-parse", "--show-toplevel"]).decode().strip())
src_dir = repo_path / "src" / "saltext" / "namecheap"
doc_dir = repo_path / "docs"

docs_by_kind = {}
changed_something = False


def _find_virtualname(path):
    tree = ast.parse(path.read_text())
    for node in ast.walk(tree):
        if isinstance(node, ast.Assign):
            for target in node.targets:
                if isinstance(target, ast.Name) and target.id == "__virtualname__":
                    if isinstance(node.value, ast.Str):
                        virtualname = node.value.s
                        break
            else:
                continue
            break
    else:
        virtualname = path.with_suffix("").name
    return virtualname


def write_module(rst_path, path, use_virtualname=True):
    if use_virtualname:
        virtualname = "``" + _find_virtualname(path) + "``"
    else:
        virtualname = make_import_path(path)
    module_contents = f"""\
{virtualname}
{'='*len(virtualname)}

.. automodule:: {make_import_path(path)}
    :members:
"""
    if not rst_path.exists() or rst_path.read_text() != module_contents:
        print(rst_path)
        rst_path.write_text(module_contents)
        return True
    return False


def write_index(index_rst, import_paths, kind):
    if kind == "utils":
        header_text = "Utilities"
        common_path = os.path.commonpath(tuple(x.replace(".", "/") for x in import_paths)).replace(
            "/", "."
        )
        if any(x == common_path for x in import_paths):
            common_path = common_path[: common_path.rfind(".")]
    else:
        header_text = (
            "execution modules" if kind.lower() == "modules" else kind.rstrip("s") + " modules"
        )
        common_path = import_paths[0][: import_paths[0].rfind(".")]
    header = f"{'_'*len(header_text)}\n{header_text.title()}\n{'_'*len(header_text)}"
    index_contents = f"""\
.. all-saltext.namecheap.{kind}:

{header}

.. currentmodule:: {common_path}

.. autosummary::
    :toctree:

{chr(10).join(sorted('    '+p[len(common_path)+1:] for p in import_paths))}
"""
    if not index_rst.exists() or index_rst.read_text() != index_contents:
        print(index_rst)
        index_rst.write_text(index_contents)
        return True
    return False


def make_import_path(path):
    if path.name == "__init__.py":
        path = path.parent
    return ".".join(path.relative_to(repo_path / "src").with_suffix("").parts)


for path in src_dir.glob("*/*.py"):
    if path.name != "__init__.py":
        kind = path.parent.name
        if kind != "utils":
            docs_by_kind.setdefault(kind, set()).add(path)

# Utils can have subdirectories, treat them separately
for path in (src_dir / "utils").rglob("*.py"):
    if path.name == "__init__.py" and not path.read_text():
        continue
    docs_by_kind.setdefault("utils", set()).add(path)

for kind in docs_by_kind:
    kind_path = doc_dir / "ref" / kind
    index_rst = kind_path / "index.rst"
    import_paths = []
    for path in sorted(docs_by_kind[kind]):
        import_path = make_import_path(path)
        import_paths.append(import_path)
        rst_path = kind_path / (import_path + ".rst")
        rst_path.parent.mkdir(parents=True, exist_ok=True)
        change = write_module(rst_path, path, use_virtualname=kind != "utils")
        changed_something = changed_something or change

    write_index(index_rst, import_paths, kind)


# Ensure pre-commit realizes we did something
if changed_something:
    exit(2)



"""
Copied/distilled from Salt doc/_ext/saltdomain.py in order to be able
to use Salt's custom doc refs.
"""


def setup(app):
    app.add_crossref_type(
        directivename="conf_master",
        rolename="conf_master",
        indextemplate="pair: %s; conf/master",
    )
    app.add_crossref_type(
        directivename="conf_minion",
        rolename="conf_minion",
        indextemplate="pair: %s; conf/minion",
    )
    return {"parallel_read_safe": True, "parallel_write_safe": True}



import pytest


@pytest.fixture(scope="package")
def master(master):
    with master.started():
        yield master


@pytest.fixture(scope="package")
def minion(minion):
    with minion.started():
        yield minion


@pytest.fixture
def salt_run_cli(master):
    return master.salt_run_cli()


@pytest.fixture
def salt_cli(master):
    return master.salt_cli()


@pytest.fixture
def salt_call_cli(minion):
    return minion.salt_call_cli()



import pytest
from salt.modules import config

from saltext.namecheap.utils import namecheap


@pytest.fixture
def namecheap_opts():
    return {
        "namecheap.name": "foo",
        "namecheap.user": "bar",
        "namecheap.key": "hunter1",
        "namecheap.client_ip": "1.2.3.4",
        "namecheap.url": "http://name.cheap",
    }


@pytest.fixture
def configure_loader_modules(namecheap_opts):
    return {
        config: {
            "__opts__": namecheap_opts,
        }
    }


def test_get_opts(namecheap_opts):
    """
    Basic test to make the test suite pass
    """
    command = "namecheap.domains.dns.getlist"
    expected = {
        "ApiUser": namecheap_opts["namecheap.name"],
        "UserName": namecheap_opts["namecheap.user"],
        "ApiKey": namecheap_opts["namecheap.key"],
        "ClientIp": namecheap_opts["namecheap.client_ip"],
        "Command": command,
    }
    opts, url = namecheap.get_opts(config.option, command)
    assert opts == expected
    assert url == namecheap_opts["namecheap.url"]
